import { config } from "./config";
import { guard } from "./guard";
import { _type } from "./symbols";
import { validate } from "./validate";
import { vality } from "./vality";
vality.string = guard("string", (val) => {
    if (typeof val === "string")
        return val;
    if (config.strict)
        return undefined;
    if (typeof val !== "number")
        return undefined;
    return val.toString();
}, {
    minLength: (val, o) => val.length >= o,
    maxLength: (val, o) => val.length <= o,
    match: (val, o) => o.test(val),
});
vality.number = guard("number", (val) => {
    if (typeof val === "number")
        return val;
    if (config.strict)
        return undefined;
    if (typeof val !== "string")
        return undefined;
    const nr = Number.parseFloat(val);
    if (Number.isNaN(nr))
        return undefined;
    return nr;
}, {
    min: (val, o) => val >= o,
    max: (val, o) => val <= o,
    integer: (val, o) => !o || val % 1 === 0,
    unsafe: (val, o) => o || (val > Number.MIN_SAFE_INTEGER && val < Number.MAX_SAFE_INTEGER),
}, {
    unsafe: false,
});
const y = ["1", 1, "true"];
const n = ["0", 0, "false"];
vality.boolean = guard("boolean", (val) => {
    if (typeof val === "boolean")
        return val;
    if (config.strict)
        return undefined;
    if (typeof val !== "string" && typeof val !== "number")
        return undefined;
    return y.indexOf(val) !== -1
        ? true
        : n.indexOf(val) !== -1
            ? false
            : undefined;
});
vality.date = guard("date", (val) => {
    if (val instanceof Date)
        return val;
    if (config.strict)
        return undefined;
    if (typeof val !== "string" && typeof val !== "number")
        return undefined;
    const date = new Date(val);
    if (Number.isNaN(date.getTime()))
        return undefined;
    return date;
}, {
    min: (val, o) => val >= o,
    max: (val, o) => val <= o,
    past: (val, o) => !o || val < new Date(),
    future: (val, o) => !o || val > new Date(),
});
vality.literal = (lit) => {
    const guardFn = (val, options) => {
        if (options.default === true) {
            if (val === undefined)
                return lit;
        }
        else {
            delete options.default;
        }
        if (val === lit)
            return lit;
        if (config.strict)
            return undefined;
        if (typeof lit === "string" && typeof val === "number")
            return val.toString() === lit ? lit : undefined;
        if (typeof lit === "number" && typeof val === "string")
            return parseFloat(val) === lit ? lit : undefined;
        if (typeof lit === "boolean" &&
            (typeof val === "string" || typeof val === "number")) {
            if (lit === true && y.indexOf(val) !== -1)
                return lit;
            if (lit === false && n.indexOf(val) !== -1)
                return lit;
        }
        return undefined;
    };
    Object.assign(guardFn, { [_type]: lit });
    return guard("literal", guardFn);
};
vality.relation = () => guard("relation", (val) => {
    return validate([
        null,
        vality.number({
            integer: true,
            min: 0,
        }),
    ], val).data;
});
vality.any = guard("any", (val) => val);
//# sourceMappingURL=guards.js.map