import { _name, _type, _validate } from "./symbols";
import { vality } from "./vality";
export function assert(val, condition) {
    if (condition === false) {
        throw new Error("Assertion failed");
    }
}
export function isValid(data) {
    return data !== undefined;
}
export function enyToGuard(eny) {
    if (Array.isArray(eny)) {
        if (eny.length === 0)
            throw new Error("Empty array short");
        if (eny.length === 1)
            return vality.array(enyToGuard(eny[0]));
        return vality.enum(...eny.map(enyToGuard));
    }
    if (typeof eny === "string" ||
        typeof eny === "number" ||
        typeof eny === "boolean" ||
        eny === null) {
        return vality.literal(eny);
    }
    if (_validate in eny)
        return eny;
    if (typeof eny === "function")
        return vality.relation(eny);
    return vality.object(eny);
}
export function enyToGuardFn(e) {
    return enyToGuard(e)[_validate];
}
export function flat(arr) {
    return [].concat(...arr);
}
export function identity(x) {
    return x;
}
export function trueFn(..._args) {
    return true;
}
export function falseFn(..._args) {
    return false;
}
export function makeValit(...[name, fn, handleOptions, defaultOptions]) {
    return (...args) => {
        const getValidateFnFromOptions = (options) => (value, path, parent) => {
            const { transform = undefined, preprocess = undefined, default: defaultValue = undefined, validate = undefined, ...optionsWithoutExtras } = { ...options };
            const optionsWithDefault = {
                ...defaultOptions,
                ...optionsWithoutExtras,
            };
            if (preprocess) {
                value = preprocess(value, options, path, parent);
            }
            const data = fn(...args)(value, options, path, parent);
            if (!data.valid) {
                if (value === undefined && defaultValue !== undefined) {
                    return { valid: true, data: defaultValue, errors: [] };
                }
                return data;
            }
            if (validate?.(data.data, options) === false) {
                return {
                    valid: false,
                    data: undefined,
                    errors: [
                        {
                            message: `vality.${name}.custom`,
                            path,
                            options,
                            value,
                        },
                    ],
                };
            }
            const origData = data.data;
            if (transform) {
                data.data = transform(data.data);
            }
            if (handleOptions === undefined)
                return data;
            const keysWithError = Object.keys(optionsWithDefault).filter((k) => handleOptions[k]?.(origData, optionsWithDefault[k], options) === false);
            if (keysWithError.length === 0)
                return data;
            return {
                valid: false,
                data: undefined,
                errors: keysWithError.map((k) => ({
                    message: k in options
                        ? `vality.${name}.options.${k}`
                        : `vality.${name}.base`,
                    options,
                    path,
                    value,
                })),
            };
        };
        function applyFnType(f) {
            f[_type] = args;
            return f;
        }
        const validate = ((options) => ({
            [_validate]: applyFnType((val, path, parent) => {
                if (typeof options === "function")
                    options = options(parent);
                return getValidateFnFromOptions(options)(val, path, parent);
            }),
            [_name]: name,
        }));
        validate[_validate] = applyFnType(getValidateFnFromOptions({}));
        validate[_name] = name;
        return validate;
    };
}
//# sourceMappingURL=utils.js.map